from swarmauri.utils._lazy_import import _lazy_import

parsers_files = [
    ("swarmauri_community.parsers.concrete.BERTEmbeddingParser", "BERTEmbeddingParser"),
    ("swarmauri_community.parsers.concrete.EntityRecognitionParser", "EntityRecognitionParser"),
    ("swarmauri_community.parsers.concrete.FitzPdfParser", "FitzPdfParser"),
    ("swarmauri_community.parsers.concrete.PyPDF2Parser", "PyPDF2Parser"),
    ("swarmauri_community.parsers.concrete.PyPDFTKParser", "PyPDFTKParser"),
    ("swarmauri_community.parsers.concrete.TextBlobNounParser", "TextBlobNounParser"),
    ("swarmauri_community.parsers.concrete.TextBlobSentenceParser", "TextBlobSentenceParser"),
]

for module_name, class_name in parsers_files:
    globals()[class_name] = _lazy_import(module_name, class_name)

__all__ = [class_name for _, class_name in parsers_files]



import logging
import chromadb
from chromadb.config import Settings

from typing import List, Union, Literal

from swarmauri.documents.concrete.Document import Document
from swarmauri_community.embeddings.concrete.Doc2VecEmbedding import Doc2VecEmbedding
from swarmauri.distances.concrete.CosineDistance import CosineDistance

from swarmauri.vector_stores.base.VectorStoreBase import VectorStoreBase
from swarmauri.vector_stores.base.VectorStoreRetrieveMixin import (
    VectorStoreRetrieveMixin,
)
from swarmauri.vector_stores.base.VectorStoreSaveLoadMixin import (
    VectorStoreSaveLoadMixin,
)
from swarmauri.vector_stores.base.VectorStorePersistentMixin import (
    VectorStorePersistentMixin,
)


class PersistentChromaDBVectorStore(
    VectorStoreSaveLoadMixin,
    VectorStoreRetrieveMixin,
    VectorStorePersistentMixin,
    VectorStoreBase,
):
    type: Literal["PersistentChromaDBVectorStore"] = "PersistentChromaDBVectorStore"

    def __init__(self, **kwargs):
        """
        Initialize the PersistentChromaDBVectorStore.

        Args:
            Args:
            **kwargs: keyword arguments.
        """
        super().__init__(**kwargs)

        self._embedder = Doc2VecEmbedding(vector_size=self.vector_size)
        self._distance = CosineDistance()

    def connect(self) -> None:
        """
        Establish a connection to ChromaDB and get or create the collection.
        """
        # settings = Settings(
        #     chroma_api_impl="chromadb.api.fastapi.FastAPI",  # Use FastAPI if LocalAPI is not supported
        #     chroma_server_host="localhost",  # Server host
        #     chroma_server_http_port=8000,  # Server port
        # )
        #
        # self.client = chromadb.Client(
        #     settings=settings,
        # )
        self.client = chromadb.Client()

        self.collection = self.client.get_or_create_collection(
            name=self.collection_name
        )
        logging.info(
            f"Connected to ChromaDB at {self.path}, collection: {self.collection_name}"
        )

    def disconnect(self) -> None:
        """
        Close the connection to ChromaDB.
        """
        if self.client:
            # Perform any necessary cleanup here
            self.client = None
            self.collection = None

    def add_document(self, document: Document) -> None:
        embedding = None
        if not document.embedding:
            self._embedder.fit([document.content])  # Fit only once
            embedding = (
                self._embedder.transform([document.content])[0].to_numpy().tolist()
            )
        else:
            embedding = document.embedding

        self.collection.add(
            ids=[document.id],
            documents=[document.content],
            embeddings=[embedding],
            metadatas=[document.metadata],
        )

    def add_documents(self, documents: List[Document]) -> None:
        ids = [doc.id for doc in documents]
        texts = [doc.content for doc in documents]

        for doc in documents:
            self._embedder.fit([doc.content])

        embeddings = [
            self._embedder.infer_vector(doc.content).value for doc in documents
        ]
        metadatas = [doc.metadata for doc in documents]
        if metadatas[0]:
            self.collection.add(
                ids=ids, documents=texts, embeddings=embeddings, metadatas=metadatas
            )
        else:
            self.collection.add(ids=ids, documents=texts, embeddings=embeddings)

    def get_document(self, doc_id: str) -> Union[Document, None]:
        results = self.collection.get(ids=[doc_id])
        if not results["metadatas"][0]:
            results["metadatas"][0] = {}
        if results["ids"]:
            document = Document(
                id=results["ids"][0],
                content=results["documents"][0],
                metadata=results["metadatas"][0],
            )
            return document
        return None

    def get_all_documents(self) -> List[Document]:
        results = self.collection.get()
        documents = [
            Document(
                id=results["ids"][idx],
                content=results["documents"][idx],
                metadata=results["metadatas"][idx],
            )
            for idx in range(len(results["ids"]))
        ]
        return documents

    def delete_document(self, doc_id: str) -> None:
        self.collection.delete(ids=[doc_id])

    def update_document(self, doc_id: str, updated_document: Document) -> None:
        document_vector = None
        # Precompute the embedding outside the update process
        if not updated_document.embedding:
            # Transform without refitting to avoid vocabulary issues
            document_vector = self._embedder.transform([updated_document.content])[0]
        else:
            document_vector = updated_document.embedding

        document_vector = document_vector.to_numpy().tolist()

        updated_document.embedding = document_vector

        self.delete_document(doc_id)
        self.add_document(updated_document)

    def clear_documents(self) -> None:
        documents = self.get_all_documents()
        doc_ids = [doc.id for doc in documents]
        self.collection.delete(ids=doc_ids)

    def document_count(self) -> int:
        return len(self.get_all_documents())

    def retrieve(self, query: str, top_k: int = 5) -> List[Document]:
        query_embedding = self._embedder.infer_vector(query).value
        # print(query_embedding)

        results = self.collection.query(
            query_embeddings=query_embedding, n_results=top_k
        )

        return [
            Document(
                id=results["ids"][0][idx],
                content=results["documents"][0][idx],
                metadata=(
                    results["metadatas"][0][idx] if results["metadatas"][0][idx] else {}
                ),
            )
            for idx in range(len(results["ids"][0]))
        ]

    # Override the model_dump_json method
    def model_dump_json(self, *args, **kwargs) -> str:
        # Call the disconnect method before serialization
        self.disconnect()

        # Now proceed with the usual JSON serialization
        return super().model_dump_json(*args, **kwargs)



from swarmauri_core.typing import SubclassUnion
from typing import List, Literal, Dict
from pydantic import Field
from swarmauri.tools.base.ToolBase import ToolBase
from swarmauri.tools.concrete.Parameter import Parameter
import nltk
from nltk.tokenize import sent_tokenize, word_tokenize

# Download required NLTK data once during module load
nltk.download("punkt", quiet=True)


class TextLengthTool(ToolBase):
    version: str = "0.1.dev1"
    parameters: List[Parameter] = Field(
        default_factory=lambda: [
            Parameter(
                name="text",
                type="string",
                description="The text to analyze for length.",
                required=True,
            )
        ]
    )
    name: str = "TextLengthTool"
    description: str = "Calculates the length of the provided text in terms of characters, words, and sentences."
    type: Literal["TextLengthTool"] = "TextLengthTool"

    def __call__(self, text: str) -> Dict[str, int]:
        """
        Calculates the length of the provided text in terms of characters, words, and sentences.

        Parameters:
            text (str): The text to analyze.

        Returns:
            dict: A dictionary containing the number of characters, words, and sentences.
        """
        return self.calculate_text_length(text)

    def calculate_text_length(self, text: str) -> Dict[str, int]:
        """
        Calculate the length of the text in characters, words, and sentences.

        Parameters:
            text (str): The text to analyze.

        Returns:
            dict: A dictionary containing:
                  - 'num_characters': Number of characters in the text.
                  - 'num_words': Number of words in the text.
                  - 'num_sentences': Number of sentences in the text.
        """
        num_characters = self.count_characters(text)
        num_words = self.count_words(text)
        num_sentences = self.count_sentences(text)

        return {
            "num_characters": num_characters,
            "num_words": num_words,
            "num_sentences": num_sentences,
        }

    def count_characters(self, text: str) -> int:
        """
        Count the number of characters in the text, excluding spaces.

        Parameters:
            text (str): The text to analyze.

        Returns:
            int: The number of characters in the text.
        """
        return len(text.replace(" ", ""))

    def count_words(self, text: str) -> int:
        """
        Count the number of words in the text.

        Parameters:
            text (str): The text to analyze.

        Returns:
            int: The number of words in the text.
        """
        words = word_tokenize(text)
        return len(words)

    def count_sentences(self, text: str) -> int:
        """
        Count the number of sentences in the text.

        Parameters:
            text (str): The text to analyze.

        Returns:
            int: The number of sentences in the text.
        """
        sentences = sent_tokenize(text)
        return len(sentences)


SubclassUnion.update(
    baseclass=ToolBase,
    type_name="TextLengthTool",
    obj=TextLengthTool,
)



from swarmauri.utils._lazy_import import _lazy_import

measurement_files = [
    ("swarmauri_community.measurements.concrete.MutualInformationMeasurement", "MutualInformationMeasurement"),
    ("swarmauri_community.measurements.concrete.TokenCountEstimatorMeasurement", "TokenCountEstimatorMeasurement"),
]

for module_name, class_name in measurement_files:
    globals()[class_name] = _lazy_import(module_name, class_name)

__all__ = [class_name for _, class_name in measurement_files]
